#include <iostream>
#include <GL/glut.h>
#include "mesh.h"
#include "objloader.h"
#include "dcimage.h"

Mesh *fromFile;

void drawMesh(Mesh* m)
{
  glVertexPointer(3,GL_FLOAT,0,m->verts);
  glTexCoordPointer(2,GL_FLOAT,0,m->uvs);
  glDrawElements (GL_TRIANGLES,m->nbrTris*3,GL_UNSIGNED_INT,m->tris);
}

// Ignore this at the moment
void textureMagic()
{
  glEnable(GL_TEXTURE_2D);
  unsigned int temp;
  glGenTextures(1,&temp); // Create The Texture
  glBindTexture(GL_TEXTURE_2D,temp);
  DCImage img;
  img.load("texture2.raw");
  glTexImage2D(GL_TEXTURE_2D,0,3,img.width,img.width,0,GL_RGB,GL_UNSIGNED_BYTE,img.data);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
}


void display(void)
{
glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT); // Clear the colour and depth buffer

glLoadIdentity(); // Clear matrix stack

static float a=0;
a+=0.5;

glTranslatef(0,0,-300);
glRotatef(a,0,1,1); // Uncomment for rotation
glColor3f(1,1,1);
drawMesh(fromFile);

glFlush(); // Makes sure that we output the model to the graphics card
glutSwapBuffers();
glutPostRedisplay();
}

// Called when a key is pressed
void key(unsigned char k, int x, int y)
{
if( k == 'q' ) exit(0);
}

void reshape(int width,int height)
{
glViewport(0,0,width,height); // Reset The Current Viewport

glMatrixMode(GL_PROJECTION); // Select The Projection Matrix
glLoadIdentity(); // Reset The Projection Matrix

// Calculate The Aspect Ratio Of The Window
gluPerspective(45.0f,(float)640/(float)480,0.1f,1000.0f);
// Always keeps the same aspect as a 640 wide and 480 high window

glMatrixMode(GL_MODELVIEW); // Select The Modelview Matrix
glLoadIdentity(); // Reset The Modelview Matrix
}

void init()
{
glClearColor(0.2,0,0.5,0);
glClearDepth(1.0); // Enables Clearing Of The Depth Buffer
glDepthFunc(GL_LESS); // The Type Of Depth Test To Do
glEnable(GL_DEPTH_TEST); // Enables Depth Testing
glShadeModel(GL_SMOOTH); // Enables Smooth Color Shading

// NEW to initialize the vertex array drawing
// used by glDrawElements,glVertexPointer and glColorPointer
glEnableClientState(GL_VERTEX_ARRAY);
glEnableClientState(GL_TEXTURE_COORD_ARRAY);

fromFile=ObjLoader::ReadMesh("spaceship.3ds");
textureMagic();

std::cout << fromFile->nbrVerts << " " << fromFile->nbrTris << std::endl;

// Initialization of triangle

}

int main(int argc, char *argv[])
{
glutInit(&argc, argv);
glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE); // We want rgb display functionality
glutInitWindowSize(640,480); // Set the window dimensions
glutInitWindowPosition(0,0); // Set the window starting point
glutCreateWindow("El Triangulo del Futuro Numero Cinco"); // Set the caption and launch the window

init();

// Last things before rendering starts
glutDisplayFunc(display); // This will be called every frame
glutReshapeFunc(reshape); // Reshape the window when something changes
glutKeyboardFunc(key); // Callback for input

glutMainLoop(); // Starts the main program

// We will not reach this point unless exit(0) is called (see function keyCB)
delete fromFile;

return 0;
}